ip_ver
------
{{ ip_ver|to_yaml }}

cfg
---
{{ cfg.cfg|to_yaml }}

bogons
------
{{ bogons.cfg|to_yaml }}

clients
-------
{{ clients.cfg|to_yaml }}

asns
----
{{ asns.cfg|to_yaml }}

irrdb_info
-------
{{ irrdb_info|parse_irrdb_info|sort(attribute="id")|to_yaml }}

roas
----
{% if roas %}
{{ roas.cfg|to_yaml }}
{% endif %}
